"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var EBootState;
(function (EBootState) {
    EBootState["Initial"] = "initial";
    EBootState["Booting"] = "booting";
    EBootState["Finished"] = "finished";
    EBootState["Failed"] = "failed";
})(EBootState = exports.EBootState || (exports.EBootState = {}));
class BootLoader {
    constructor(logger) {
        this._state = EBootState.Initial;
        this._bootFunctionObjects = [];
        this._afterBootFunctionObjects = [];
        this._logger = logger;
    }
    getBootState() {
        return this._state;
    }
    isBooting() {
        return this._state === EBootState.Booting;
    }
    hasBooted() {
        return this._state === EBootState.Finished;
    }
    addBootFunction(name, fn) {
        this._logger.silly("addBootFunction", name);
        this._bootFunctionObjects.push({ name, fn: fn });
    }
    onBootReady(name, fn) {
        this._logger.silly("onBootReady", name);
        if (this._state === EBootState.Finished) {
            return fn();
        }
        this._afterBootFunctionObjects.push({ name, fn });
    }
    getReadyPromise() {
        return new Promise((resolve, reject) => {
            // eslint-disable-next-line @typescript-eslint/no-floating-promises
            this.onBootReady("BootLoader is ready", () => resolve());
            this._failedBootPromiseRejection = reject;
        });
    }
    async boot() {
        this._state = EBootState.Booting;
        this._logger.debug("booting functions starting...");
        try {
            for (const fnObj of this._bootFunctionObjects) {
                this._logger.debug("booting function start: ", fnObj.name);
                await fnObj.fn(this);
                this._logger.debug("booting function end: ", fnObj.name);
            }
            this._logger.debug("... finished booting function.");
            this._state = EBootState.Finished;
            for (const fnObj of this._afterBootFunctionObjects) {
                this._logger.debug("after booting hook function start: ", fnObj.name);
                await fnObj.fn(this);
                this._logger.debug("after booting hook function end: ", fnObj.name);
            }
        }
        catch (err) {
            this._logger.error("BootLoader caught Error:", err);
            this._state = EBootState.Failed;
            if (this._failedBootPromiseRejection != null) {
                this._failedBootPromiseRejection(err);
            }
            throw err;
        }
    }
}
exports.BootLoader = BootLoader;
//# sourceMappingURL=index.js.map