"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const errors_1 = require("../errors");
class ConfigMergeHelper {
    static checkForMissingConfigProperties(moduleName, config) {
        const missingProperties = [];
        this._deepForEach(config, (key, val, nestedPath) => {
            if (val == null) {
                missingProperties.push(nestedPath);
            }
        });
        if (missingProperties.length > 0) {
            throw new errors_1.MissingConfigPropertiesError(moduleName, missingProperties);
        }
    }
    static getProcessEnvironmentConfig(moduleName) {
        const processEnvironmentConfig = {};
        Object.entries(process.env).forEach(([key, value]) => {
            if (value != null) {
                const parsedValue = this._parseTrueAndFalseToBooleans(value);
                _.set(processEnvironmentConfig, key, parsedValue);
            }
        });
        const processEnvironmentConfigOfModule = processEnvironmentConfig[moduleName] || {};
        return processEnvironmentConfigOfModule;
    }
    static _parseTrueAndFalseToBooleans(value) {
        const lowerCaseValue = value.toString().toLowerCase();
        return lowerCaseValue === "true";
    }
    static _deepForEach(obj, callback, nestedPath = "") {
        Object.entries(obj).map((entry) => {
            const newPath = `${nestedPath}${entry[0]}.`;
            return typeof entry[1] === "object" && entry[1] != null
                ? this._deepForEach(entry[1], callback, newPath)
                : callback(entry[0], entry[1], newPath.slice(0, -1));
        });
    }
}
exports.default = ConfigMergeHelper;
//# sourceMappingURL=ConfigMergeHelper.js.map