"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const path = require("path");
class EnvironmentBuilder {
    static buildEnvironment(NODE_ENV, namespace) {
        const frameworkVersion = require("../../../package.json").version;
        const applicationRootPath = path.dirname(require.main?.filename ?? "");
        const { applicationName, applicationVersion, } = this._getApplicationNameAndVersion(applicationRootPath);
        const nodeId = this._generateNodeId(EnvironmentBuilder.nodeIdLength);
        return {
            frameworkVersion,
            NODE_ENV,
            name: applicationName,
            path: applicationRootPath,
            version: applicationVersion,
            namespace,
            nodeId,
        };
    }
    static _getApplicationNameAndVersion(applicationRootPath) {
        return {
            applicationName: process.env.npm_package_name,
            applicationVersion: process.env.npm_package_version,
        };
    }
    static _generateNodeId(length) {
        return crypto_1.randomBytes(20 + length)
            .toString("hex")
            .substr(5, length);
    }
}
exports.default = EnvironmentBuilder;
EnvironmentBuilder.nodeIdLength = 6;
//# sourceMappingURL=EnvironmentBuilder.js.map