"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const _ = require("lodash");
const ConfigMergeHelper_1 = require("./helpers/ConfigMergeHelper");
const EnvironmentBuilder_1 = require("./helpers/EnvironmentBuilder");
class ConfigManager {
    constructor() {
        this._applicationConfig = {};
        this._config = {
            Config: { namespace: "soniq" },
        };
        this._NODE_ENV = process.env?.NODE_ENV ?? "production";
        this._applicationConfig = this._loadApplicationConfig();
        this.registerConfig(this.constructor.name, `${__dirname}/../config`);
        const namespace = this._config.Config.namespace;
        this.ENVIRONMENT = EnvironmentBuilder_1.default.buildEnvironment(this._NODE_ENV, namespace);
    }
    _loadApplicationConfig() {
        const applicationConfigFolderPath = `${path.dirname(require.main?.filename ?? "")}/config`;
        return this._getConfigFromConfigFiles(applicationConfigFolderPath);
    }
    _getConfigFromConfigFiles(configDirectory) {
        const defaultConfigPath = `${configDirectory}/default.js`;
        const environmentConfigPath = `${configDirectory}/${this._NODE_ENV}.js`;
        let defaultConfig;
        try {
            defaultConfig = require(defaultConfigPath);
        }
        catch (err) {
            defaultConfig = {};
            console.warn(`config.default.loading.error.not.found: Continuing without app configuration. Was expecting file at ${defaultConfigPath}`);
        }
        let environmentConfig;
        try {
            environmentConfig = require(environmentConfigPath);
        }
        catch (err) {
            environmentConfig = {};
        }
        return _.defaultsDeep(environmentConfig, defaultConfig);
    }
    _applyConfigModule(name, baseConfigModule) {
        if (name in this._config)
            return;
        const applicationConfigOfModule = this._applicationConfig[name] || {};
        const processEnvironmentConfigOfModule = ConfigMergeHelper_1.default.getProcessEnvironmentConfig(name);
        const configModule = _.defaultsDeep(processEnvironmentConfigOfModule, applicationConfigOfModule, baseConfigModule);
        ConfigMergeHelper_1.default.checkForMissingConfigProperties(name, configModule);
        this._config[name] = configModule;
    }
    registerConfig(name, configDirectory) {
        const baseConfigModule = this._getConfigFromConfigFiles(configDirectory);
        this._applyConfigModule(name, baseConfigModule);
        return this.getConfig(name);
    }
    registerApplicationConfigModule(name) {
        this._applyConfigModule(name, {});
        return this.getConfig(name);
    }
    getConfig(name) {
        if (!_.has(this._config, name)) {
            throw new Error(`config.module.not.found module name: ${name}`);
        }
        return _.cloneDeep(this._config[name]);
    }
    dangerouslyGetWholeConfig() {
        return _.cloneDeep(this._config);
    }
}
exports.ConfigManager = ConfigManager;
//# sourceMappingURL=index.js.map